# author: abaj
# date: 30.08.2020
# (c) AMS AG

# XML generator for pinmap file in Digital Pattern Editor

from xml.dom import minidom
import os
#import lxml.etree as etree
import xml.dom.minidom
import xml.etree.ElementTree as etr
from xml.dom.minidom import parse
import logging

pinmapSourceLogger = logging.getLogger(__name__)

#Function used only as helper class during the implementation and is only used to get the elements from the list.
#Parameter:
#@list_ - specifies the list_ from which we are asking for the elements 
def getElemFromList(list_):
    """Function used only as helper class during the implementation and is only used to get the elements from the list.

    Args:
        list_ (list): specifies the list_ from which we are asking for the elements 

    Returns:
        list: list of elements
    """
    list_to_be_returned = list()
    for s in list_:
        list_to_be_returned.append(s)
    return list_to_be_returned

#Function used only as helper class during the implementation and is only used to help during the creation of PinGroups.
#Because everything is an string, therefore a group of pins named PinGroup1 consists of one or more pins.
#If group consists of more than one pin, the pins are defined in a such way(Pin1,Pin2,Pin3) and therefore 
#we need to split it by "," to get the correct amount of pins that are in our group.
#Parameter:
#@str_to_convert - specifies the string, from which are we extracting the pins         
def convStrToList(str_to_convert):
    """Function used only as helper class during the implementation and is only used to help during the creation of PinGroups. Because everything is an string, therefore a group of pins named PinGroup1 consists of one or more pins. If group consists of more than one pin, the pins are defined in a such way(Pin1,Pin2,Pin3) and therefore we need to split it by "," to get the correct amount of pins that are in our group.

    Args:
        str_to_convert (string): string containing the comma seperated list of pins

    Returns:
        list: list containing strings of the pin names
    """
    new_list = list()
    if ',' in str_to_convert:
        splitted_string = str_to_convert.split(',')
        for s in splitted_string:
            new_list.append(s)
    else:
        new_list.append(str_to_convert)
    return new_list

#Class used to create a whole Pinmap, whole logic of the file is in this class implemented, except for few helper functions.
class Pinmap:
    # XMLHeader stuff, always like this, nothing special
    import logging
    pinmapSourceLogger.debug("Building up an XML framework header")
    root = minidom.Document()
    xml = root.createElement('PinMap')
    xml.setAttribute('schemaVersion', '1.5')
    xml.setAttribute('xmlns', 'http://www.ni.com/TestStand/SemiconductorModule/PinMap.xsd')
    xml.setAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance')
    root.appendChild(xml)
    productChild = root.createElement('Instruments')
    xml.appendChild(productChild)

    pinChild = root.createElement('Pins')
    xml.appendChild(pinChild)

    pingroupChild = root.createElement('PinGroups')
    xml.appendChild(pingroupChild)

    sitegroupChild = root.createElement('Sites')
    xml.appendChild(sitegroupChild)

    connectionsChild = root.createElement('Connections')
    xml.appendChild(connectionsChild)
    pinmapSourceLogger.debug("Building up an XML framework header finished")
    list_of_instruments = list()
    pins = list()
    pingroup = list()

    #Function used to create consolehandler - logging relevant!    
    #def createConsoleHandler(self):
    #    self.consoleHandler = logging.StreamHandler()

    #Function used to create connection.
    #Parameters:
    #@self specifies the instance of the class that is currently calling this function.
    #@pin specifies the name of the pin.
    #@site_number specifies the number of the site/sites.
    #@instrument_name specifies the name of the instrument.
    #@channel_number specifies the channel number of the choosen instrument.
    def addConnection(self, pin, site_number, instrument_name, channel_number):
        """adds a connection between a channel on an instrument and a pin name

        Args:
            pin (string): the channels pin name 
            site_number (int): number of the site (usually 0)
            instrument_name (string): name of the instrument where this pin shall be connected to
            channel_number (int): number of the channel on the instrument
        """
        connection = Pinmap.root.createElement('Connection')
        connection.setAttribute('pin', '%s'%(pin))
        connection.setAttribute('siteNumber', '%s'%site_number)
        connection.setAttribute('instrument', '%s'%instrument_name)
        connection.setAttribute('channel', '%s'%channel_number)
        Pinmap.connectionsChild.appendChild(connection)

    #Function used to create the site.
    #Parameters:
    #@self specifies the instance of the class that is currently calling this function.
    #@number_of_site specifies the number of the site that we are currently creating.
    def addSites(self, number_of_site):
        """Add a test site. There has to be at least one site where a DUT is tested

        Args:
            number_of_site (int): the site number
        """
        site = Pinmap.root.createElement('Site')
        site.setAttribute('siteNumber', '%s'%number_of_site)
        Pinmap.sitegroupChild.appendChild(site)

    #Function used to create the pingroup.
    #@self specifies the instance of the class that is currently calling this function.
    #@name specifies name of the group.
    #@list_of_pins specifies the list of the pins that this group consist of.
    def addPinGroup(self, name, list_of_pins):
        """ Add a list of pins that should be treated as a group

        Args:
            name (string): name of the pin group
            list_of_pins (string): comma-seperated list of the pins that shall be used for this group
        """
        conv_list_of_strings = convStrToList(list_of_pins)
        pin_group = Pinmap.root.createElement('PinGroup')
        pin_group.setAttribute('name', '%s'%(name))
        Pinmap.pingroupChild.appendChild(pin_group)
        for s in conv_list_of_strings:
            if s in getElemFromList(Pinmap.pins):
                pin_to_add = Pinmap.root.createElement('PinReference')
                pin_to_add.setAttribute('pin', '%s'%s)
                pin_group.appendChild(pin_to_add)
            elif s not in getElemFromList(Pinmap.pins):
                pinmapSourceLogger.debug("Pin with name %s does not exist!"%(s))

    #Function used to create the pin.
    #@self specifies the instance of the class that is currently calling this function.
    #@pin_type specifies the type of the pin.
    #@pin_name specifies the name of the pin.
    def addPin(self, pin_type, pin_name):
        """adds a pin to the pinmap

        Args:
            pin_type (string): The Type of the pin. Possible Values: 'DUTPin', "SystemPin"
            pin_name (string): The name of the pin
        """
        pin = Pinmap.root.createElement('%s'%(pin_type))
        pin.setAttribute('name', '%s'%(pin_name))
        Pinmap.pins.append('%s'%pin_name)
        Pinmap.pinChild.appendChild(pin)

    #Function used to add the instrument.
    #@self specifies the instance of the class that is currently calling this function.
    #@instrument_type_ specifies type of the instrument.
    #@instrument_name specifies the name of the instrument.
    #@number_of_channels the number of channel that this instrument posess.
    def addInstrument(self, instrument_type_, instrument_name, number_of_channels):
        """Function used to add an instrument to the pinmap

        Args:
            instrument_type_ (string): the type of the instrument. Possible Values: 'NIDigitalPatternInstrument', 'NIDCPowerInstrument', ... (refer to Digital Pattern Editor Help)
            instrument_name (string): The name of the Instrument (like in NImax)
            number_of_channels (int): The number of channels that this instrument provides
        """
        instrument_type = Pinmap.root.createElement('%s'%(instrument_type_))
        instrument_type.setAttribute('name', '%s'%(instrument_name))
        instrument_type.setAttribute('numberOfChannels', '%s'%(number_of_channels))
        Pinmap.productChild.appendChild(instrument_type)
        Pinmap.list_of_instruments.append(instrument_type)

    #Function used to generate XML digilevel file 
    #Parameters:
    #@self specifies the instance of the class which actually generates its XML file
    #@name_of_file specifies the name of the file that we want to generate 
    def generateXML(self, filepath = ""):
        """generates and writes the xml file for the created PinLevelSheet Object

        Args:
            filepath (str): specify the name/path of the digilevels file. Skipping if no name is given 
        """
        if(filepath != "" and filepath[-7:] == ".pinmap"):
            pinmapSourceLogger.debug("Writing an file")
            save_path_file = "%s"%(filepath)
            with open(save_path_file, "w") as f:
                Pinmap.root.writexml(f, "\n", "  ", encoding="utf-8")

'''
#test is used to check the implementation of the whole file
def test():
    p3 = Pinmap()
    p3.pinmapSourceLogger.setLevel(logging.DEBUG)
    p3.createConsoleHandler()
    p3.consoleHandler.setLevel(logging.DEBUG)
    p3.pinmapSourceLogger.addHandler(p3.consoleHandler)
    p3.pinmapSourceLogger.debug("-----------------------STARTING TEST------------------")
    p3.pinmapSourceLogger.debug("Adding Instruments")
    p3.addInstrument("NIDigitalPatternInstrument", "PXI1Slot2", 32)
    p3.addInstrument("NIDigitalPatternInstrument", "PXI1Slot3", 32)
    p3.pinmapSourceLogger.debug("Adding Pins")
    p3.addPin("DUTPin", "DUTPin1")
    p3.addPin("DUTPin", "DUTPin2")
    p3.addPin("DUTPin", "DUTPin3")
    p3.addPin("DUTPin", "DUTPin4")
    p3.addPin("DUTPin", "DUTPin5")
    p3.addPin("DUTPin", "DUTPin6")
    p3.addPin("DUTPin", "DUTPin7")
    p3.addPin("DUTPin", "DUTPin8")
    p3.pinmapSourceLogger.debug("Creating groups of added pins")
    p3.addPinGroup("PinGroup1", "DUTPin1,DUTPin2,DUTPin3,DUTPin4,DUTPin5,DUTPin6,DUTPin7,DUTPin8")
    p3.addPinGroup("PinGroup2", "DUTPin1,DUTPin2,DUTPin3,DUTPin4,DUTPin5,DUTPin6,DUTPin7,DUTPin8")
    p3.addPinGroup("PinGroup3", "DUTPin1,DUTPin2,DUTPin3,DUTPin4,DUTPin5,DUTPin6,DUTPin7,DUTPin8")
    p3.addPinGroup("PinGroup4", "DUTPin1,DUTPin2,DUTPin3,DUTPin4,DUTPin5,DUTPin6,DUTPin7,DUTPin8")
    p3.pinmapSourceLogger.debug("Creating sites")
    p3.addSites("0")
    p3.addSites("1")
    p3.pinmapSourceLogger.debug("Creating connections between pins, sites, instrument and channels on the instrument")
    p3.addConnection("DUTPin1", "0,1", "PXI1Slot2", "17")
    p3.addConnection("DUTPin2", "0", "PXI1Slot2", "18")
    p3.addConnection("DUTPin2", "1", "PXI1Slot3", "1")
    p3.addConnection("DUTPin3", "0", "PXI1Slot2", "2")
    p3.addConnection("DUTPin3", "1", "PXI1Slot3", "2")
    p3.addConnection("DUTPin4", "0", "PXI1Slot2", "3")
    p3.addConnection("DUTPin4", "1", "PXI1Slot3", "3")
    p3.addConnection("DUTPin5", "0", "PXI1Slot2", "4")
    p3.addConnection("DUTPin5", "1", "PXI1Slot3", "4")
    p3.addConnection("DUTPin6", "0", "PXI1Slot2", "5")
    p3.addConnection("DUTPin6", "1", "PXI1Slot3", "5")
    p3.addConnection("DUTPin7", "0", "PXI1Slot2", "6")
    p3.addConnection("DUTPin7", "1", "PXI1Slot3", "6")
    p3.addConnection("DUTPin8", "0", "PXI1Slot2", "7")
    p3.addConnection("DUTPin8", "1", "PXI1Slot3", "7")
    p3.pinmapSourceLogger.debug("-----------------------ENDING TEST------------------")
    p3.pinmapSourceLogger.debug("Generating file")
    p3.generateXML("test_2.pinmap")
'''
